@@ -3,33 +3,37 @@
 //  We allow or deny clients according to their IP address. It may keep 
 //  spammers and idiots away, but won't stop a real attacker for more 
 //  than a heartbeat.
-
+//
+//  CZMQ APIv3
+//
+//  For more info see: http://hintjens.com/blog:49#toc3
+//
 #include <czmq.h>
 
 int main (void) 
 {
-    //  Create context
-    zctx_t *ctx = zctx_new ();
-    
     //  Start an authentication engine for this context. This engine
     //  allows or denies incoming connections (talking to the libzmq
     //  core over a protocol called ZAP).
-    zauth_t *auth = zauth_new (ctx);
+    zactor_t *auth = zactor_new(zauth,NULL);
     
     //  Get some indication of what the authenticator is deciding
-    zauth_set_verbose (auth, true);
+    zstr_send (auth,"VERBOSE");
+    zsock_wait (auth);
     
     //  Whitelist our address; any other address will be rejected
-    zauth_allow (auth, "127.0.0.1");
-        
+    //  Add as much address as argument as you like before the NULL-Argument
+    //  e.g. zstr_sendx (auth,"127.0.0.1","192.168.1.20",NULL);
+    zstr_sendx (auth,"PLAIN","127.0.0.1",NULL);    
+    zsock_wait (auth);
+    
     //  Create and bind server socket
-    void *server = zsocket_new (ctx, ZMQ_PUSH);
-    zsocket_set_zap_domain (server, "global");
-    zsocket_bind (server, "tcp://*:9000");
+    zsock_t *server = zsock_new (ZMQ_PUSH);
+    zsock_bind (server, "tcp://*:9000");
 
     //  Create and connect client socket
-    void *client = zsocket_new (ctx, ZMQ_PULL);
-    zsocket_connect (client, "tcp://127.0.0.1:9000");
+    zsock_t *client = zsock_new (ZMQ_PULL);
+    zsock_connect (client, "tcp://127.0.0.1:9000");
     
     //  Send a single message from server to client
     zstr_send (server, "Hello");
@@ -38,7 +42,8 @@ int main (void)
     free (message);
     puts ("Strawhouse test OK");
 
-    zauth_destroy (&auth);
-    zctx_destroy (&ctx);
+    zactor_destroy (&auth);
+    zsock_destroy (&client);
+    zsock_destroy (&server);    
     return 0;
 }
