@@ -39,11 +39,13 @@
  *  __UTYPE_HPUX        HP/UX
  *  __UTYPE_ANDROID     Android
  *  __UTYPE_LINUX       Linux
+ *  __UTYPE_GNU         GNU/Hurd
  *  __UTYPE_MIPS        MIPS (BSD 4.3/System V mixture)
  *  __UTYPE_NETBSD      NetBSD
  *  __UTYPE_NEXT        NeXT
  *  __UTYPE_OPENBSD     OpenBSD
  *  __UTYPE_OSX         Apple Macintosh OS X
+ *  __UTYPE_IOS         Apple iOS
  *  __UTYPE_QNX         QNX
  *  __UTYPE_IRIX        Silicon Graphics IRIX
  *  __UTYPE_SINIX       SINIX-N (Siemens-Nixdorf Unix)
@@ -78,6 +80,7 @@
 #   define __MSDOS__
 //  Stop cheeky warnings about "deprecated" functions like fopen
 #   if _MSC_VER >= 1500
+#       undef  _CRT_SECURE_NO_DEPRECATE
 #       define _CRT_SECURE_NO_DEPRECATE
 #       pragma warning(disable: 4996)
 #   endif
@@ -146,15 +149,18 @@
 #elif (defined (__ANDROID__))
 #   define __UTYPE_ANDROID
 #   define __UNIX__
-#elif (defined (LINUX) || defined (linux))
+#elif (defined (LINUX) || defined (linux) || defined (__linux__))
 #   define __UTYPE_LINUX
 #   define __UNIX__
 #   ifndef __NO_CTYPE
 #   define __NO_CTYPE                   //  Suppress warnings on tolower()
 #   endif
-#   ifndef _BSD_SOURCE
-#   define _BSD_SOURCE                  //  Include stuff from 4.3 BSD Unix
+#   ifndef _DEFAULT_SOURCE
+#   define _DEFAULT_SOURCE                  //  Include stuff from 4.3 BSD Unix
 #   endif
+#elif (defined (__GNU__))
+#   define __UTYPE_GNU
+#   define __UNIX__
 #elif (defined (Mips))
 #   define __UTYPE_MIPS
 #   define __UNIX__
@@ -168,8 +174,13 @@
 #   define __UTYPE_OPENBSD
 #   define __UNIX__
 #elif (defined (APPLE) || defined (__APPLE__))
-#   define __UTYPE_OSX
+#   include <TargetConditionals.h>
 #   define __UNIX__
+#   if TARGET_OS_IPHONE || TARGET_IPHONE_SIMULATOR
+#      define __UTYPE_IOS
+#   else
+#      define __UTYPE_OSX
+#   endif
 #elif (defined (NeXT))
 #   define __UTYPE_NEXT
 #   define __UNIX__
@@ -182,7 +193,7 @@
 #elif (defined (sinix))
 #   define __UTYPE_SINIX
 #   define __UNIX__
-#elif (defined (SOLARIS) || defined (__SRV4))
+#elif (defined (SOLARIS) || defined (__SVR4)) || defined (SVR4)
 #   define __UTYPE_SUNSOLARIS
 #   define __UNIX__
 #elif (defined (SUNOS) || defined (SUN) || defined (sun))
@@ -198,6 +209,13 @@
 #   define __UTYPE_GENERIC
 #endif
 
+//- Always include ZeroMQ headers -------------------------------------------
+
+#include "zmq.h"
+#if (ZMQ_VERSION < ZMQ_MAKE_VERSION (4, 2, 0))
+#   include "zmq_utils.h"
+#endif
+
 //- Standard ANSI include files ---------------------------------------------
 
 #include <ctype.h>
@@ -219,9 +237,9 @@
 
 #if (defined (__MSDOS__))
 #   if (defined (__WINDOWS__))
-#       if (_WIN32_WINNT < 0x0501)
+#       if (_WIN32_WINNT < 0x0600)
 #           undef _WIN32_WINNT
-#           define _WIN32_WINNT 0x0501
+#           define _WIN32_WINNT 0x0600
 #       endif
 #       if (!defined (FD_SETSIZE))
 #           define FD_SETSIZE 1024      //  Max. filehandles/sockets
@@ -288,9 +306,19 @@
      || (defined (_POSIX_VERSION)  && (_POSIX_VERSION  >= 199309L)))
 #       include <sched.h>
 #   endif
+#   if (defined (__UTYPE_OSX) || defined (__UTYPE_IOS))
+#       include <mach/clock.h>
+#       include <mach/mach.h>           //  For monotonic clocks
+#   endif
 #   if (defined (__UTYPE_OSX))
 #       include <crt_externs.h>         //  For _NSGetEnviron()
 #   endif
+#   if (defined (__UTYPE_ANDROID))
+#       include <android/log.h>
+#   endif
+#   if (defined (__UTYPE_LINUX) && defined (HAVE_LIBSYSTEMD))
+#       include <systemd/sd-daemon.h>
+#   endif
 #endif
 
 #if (defined (__VMS__))
@@ -341,14 +369,18 @@
 #   endif
 #endif
 
-//  Add missing defines for Android
-#ifdef __UTYPE_ANDROID
-#   ifndef S_IREAD
-#       define S_IREAD S_IRUSR
-#   endif
-#   ifndef S_IWRITE
-#       define S_IWRITE S_IWUSR
-#   endif
+//  Add missing defines for non-POSIX systems
+#ifndef S_IRUSR
+#   define S_IRUSR S_IREAD
+#endif
+#ifndef S_IWUSR
+#   define S_IWUSR S_IWRITE
+#endif
+#ifndef S_ISDIR
+#   define S_ISDIR(m) (((m) & S_IFDIR) != 0)
+#endif
+#ifndef S_ISREG
+#   define S_ISREG(m) (((m) & S_IFREG) != 0)
 #endif
 
 
@@ -369,7 +401,19 @@
 typedef unsigned char   byte;           //  Single unsigned byte = 8 bits
 typedef unsigned short  dbyte;          //  Double byte = 16 bits
 typedef unsigned int    qbyte;          //  Quad byte = 32 bits
-typedef struct sockaddr_in inaddr_t;    //  Internet socket address structure
+typedef struct sockaddr_in  inaddr_t;   //  Internet socket address structure
+typedef struct sockaddr_in6 in6addr_t;  //  Internet 6 socket address structure
+
+// Common structure to hold inaddr_t and in6addr_t with length
+typedef struct {
+    union {
+        inaddr_t __addr;          //  IPv4 address
+        in6addr_t __addr6;        //  IPv6 address
+    } __inaddr_u;
+#define ipv4addr   __inaddr_u.__addr
+#define ipv6addr   __inaddr_u.__addr6
+    int inaddrlen;
+} inaddr_storage_t;
 
 //- Inevitable macros -------------------------------------------------------
 
@@ -377,15 +421,21 @@ typedef struct sockaddr_in inaddr_t;    //  Internet socket address structure
 #define strneq(s1,s2)   (strcmp ((s1), (s2)))
 
 //  Provide random number from 0..(num-1)
+//  Note that (at least in Solaris) while rand() returns an int limited by
+//  RAND_MAX, random() returns a 32-bit value all filled with random bits.
 #if (defined (__WINDOWS__)) || (defined (__UTYPE_IBMAIX)) \
- || (defined (__UTYPE_HPUX)) || (defined (__UTYPE_SUNOS))
+ || (defined (__UTYPE_HPUX)) || (defined (__UTYPE_SUNOS)) || (defined (__UTYPE_SOLARIS))
 #   define randof(num)  (int) ((float) (num) * rand () / (RAND_MAX + 1.0))
 #else
-#   define randof(num)  (int) ((float) (num) * random () / (RAND_MAX + 1.0))
+# if defined(RAND_MAX)
+#   define randof(num)  (int) ((float) (num) * (random () % RAND_MAX) / (RAND_MAX + 1.0))
+# else
+#   define randof(num)  (int) ((float) (num) * (uint32_t)random () / (UINT32_MAX + 1.0))
+# endif
 #endif
 
 // Windows MSVS doesn't have stdbool
-#if (defined (__WINDOWS__))
+#if (defined (_MSC_VER))
 #   if (!defined (__cplusplus) && (!defined (true)))
 #       define true 1
 #       define false 0
@@ -405,6 +455,7 @@ typedef struct sockaddr_in inaddr_t;    //  Internet socket address structure
 #       define inline __inline
 #   endif
 #   define strtoull _strtoui64
+#   define atoll _atoi64
 #   define srandom srand
 #   define TIMEZONE _timezone
 #   if (!defined (__MINGW32__))
@@ -415,11 +466,47 @@ typedef struct sockaddr_in inaddr_t;    //  Internet socket address structure
     typedef unsigned int  uint;
 #   if (!defined (__MINGW32__))
     typedef int mode_t;
-    typedef long ssize_t;
+#     if !defined (_SSIZE_T_DEFINED)
+typedef intptr_t ssize_t;
+#       define _SSIZE_T_DEFINED
+#     endif
+#   endif
+#   if ((!defined (__MINGW32__) \
+    || (defined (__MINGW32__) && defined (__IS_64BIT__))) \
+    && !defined (ZMQ_DEFINED_STDINT))
+    typedef __int8 int8_t;
+    typedef __int16 int16_t;
     typedef __int32 int32_t;
     typedef __int64 int64_t;
+    typedef unsigned __int8 uint8_t;
+    typedef unsigned __int16 uint16_t;
     typedef unsigned __int32 uint32_t;
     typedef unsigned __int64 uint64_t;
+#   endif
+    typedef uint32_t in_addr_t;
+#   if (!defined (PRId8))
+#       define PRId8    "d"
+#   endif
+#   if (!defined (PRId16))
+#       define PRId16   "d"
+#   endif
+#   if (!defined (PRId32))
+#       define PRId32   "d"
+#   endif
+#   if (!defined (PRId64))
+#       define PRId64   "I64d"
+#   endif
+#   if (!defined (PRIu8))
+#       define PRIu8    "u"
+#   endif
+#   if (!defined (PRIu16))
+#       define PRIu16   "u"
+#   endif
+#   if (!defined (PRIu32))
+#       define PRIu32   "u"
+#   endif
+#   if (!defined (PRIu64))
+#       define PRIu64   "I64u"
 #   endif
 #   if (!defined (va_copy))
     //  MSVC does not support C99's va_copy so we use a regular assignment
@@ -428,22 +515,29 @@ typedef struct sockaddr_in inaddr_t;    //  Internet socket address structure
 #elif (defined (__UTYPE_OSX))
     typedef unsigned long ulong;
     typedef unsigned int uint;
+    //  This fixes header-order dependence problem with some Linux versions
+#elif (defined (__UTYPE_LINUX))
+#   if (__STDC_VERSION__ >= 199901L && !defined (__USE_MISC))
+    typedef unsigned int uint;
+#   endif
 #endif
 
-//- Error reporting ---------------------------------------------------------
+//- Non-portable declaration specifiers -------------------------------------
+
+//  For thread-local storage
+#if defined (__WINDOWS__)
+#   define CZMQ_THREADLS __declspec(thread)
+#else
+#   define CZMQ_THREADLS __thread
+#endif
 
 //  Replacement for malloc() which asserts if we run out of heap, and
 //  which zeroes the allocated block.
 static inline void *
-    safe_malloc (
-    size_t size,
-    const char *file,
-    unsigned line)
+safe_malloc (size_t size, const char *file, unsigned line)
 {
-    void
-        *mem;
-
-    mem = calloc (1, size);
+//     printf ("%s:%u %08d\n", file, line, (int) size);
+    void *mem = calloc (1, size);
     if (mem == NULL) {
         fprintf (stderr, "FATAL ERROR at %s:%u\n", file, line);
         fprintf (stderr, "OUT OF MEMORY (malloc returned NULL)\n");
@@ -454,11 +548,11 @@ static inline void *
 }
 
 //  Define _ZMALLOC_DEBUG if you need to trace memory leaks using e.g. mtrace,
-//  otherwise all allocations will claim to come from zfl_prelude.h.  For best
+//  otherwise all allocations will claim to come from czmq_prelude.h. For best
 //  results, compile all classes so you see dangling object allocations.
 //  _ZMALLOC_PEDANTIC does the same thing, but its intention is to propagate
 //  out of memory condition back up the call stack.
-#if defined _ZMALLOC_DEBUG || _ZMALLOC_PEDANTIC
+#if defined (_ZMALLOC_DEBUG) || defined (_ZMALLOC_PEDANTIC)
 #   define zmalloc(size) calloc(1,(size))
 #else
 #   define zmalloc(size) safe_malloc((size), __FILE__, __LINE__)
@@ -471,10 +565,21 @@ static inline void *
 #   define CHECK_PRINTF(a)
 #endif
 
-//- Socket header files -----------------------------------------------------
+//  Lets us write code that compiles both on Windows and normal platforms
+#if !defined (__WINDOWS__)
+typedef int SOCKET;
+#   define closesocket      close
+#   define INVALID_SOCKET   -1
+#   define SOCKET_ERROR     -1
+#   define O_BINARY         0
+#endif
+
+//- Include non-portable header files based on platform.h -------------------
 
 #if defined (HAVE_LINUX_WIRELESS_H)
 #   include <linux/wireless.h>
+//  This would normally come from net/if.h
+unsigned int if_nametoindex (const char *ifname);
 #else
 #   if defined (HAVE_NET_IF_H)
 #       include <net/if.h>
@@ -483,43 +588,43 @@ static inline void *
 #       include <net/if_media.h>
 #   endif
 #endif
-//  Lets us write code that compiles both on Windows and normal platforms
-#if !defined (__WINDOWS__)
-typedef int SOCKET;
-#   define closesocket close
-#   define INVALID_SOCKET -1
-#   define SOCKET_ERROR -1
+
+#if defined (__WINDOWS__) && !defined (HAVE_UUID)
+#   define HAVE_UUID 1
 #endif
-
-//- DLL exports -------------------------------------------------------------
-
-#if defined (__WINDOWS__)
-#   if defined LIBCZMQ_STATIC
-#       define CZMQ_EXPORT
-#   elif defined LIBCZMQ_EXPORTS
-#       define CZMQ_EXPORT __declspec(dllexport)
-#   else
-#       define CZMQ_EXPORT __declspec(dllimport)
+#if defined (__UTYPE_OSX) && !defined (HAVE_UUID)
+#   define HAVE_UUID 1
+#endif
+#if defined (HAVE_UUID)
+#   if defined (__UTYPE_FREEBSD) || defined (__UTYPE_NETBSD)
+#       include <uuid.h>
+#   elif defined __UTYPE_HPUX
+#       include <dce/uuid.h>
+#   elif defined (__UNIX__)
+#       include <uuid/uuid.h>
 #   endif
-#else
-#   define CZMQ_EXPORT
 #endif
 
-//- Always include ZeroMQ header file ---------------------------------------
-
-#include "zmq.h"
-#include "zmq_utils.h"
+//  ZMQ compatibility macros
 
 #if ZMQ_VERSION_MAJOR == 4
 #   define ZMQ_POLL_MSEC    1           //  zmq_poll is msec
 
 #elif ZMQ_VERSION_MAJOR == 3
 #   define ZMQ_POLL_MSEC    1           //  zmq_poll is msec
+#   if  ZMQ_VERSION_MINOR < 2
+#       define zmq_ctx_new  zmq_init
+#   endif
+#   define zmq_ctx_term     zmq_term
 
 #elif ZMQ_VERSION_MAJOR == 2
 #   define ZMQ_POLL_MSEC    1000        //  zmq_poll is usec
 #   define zmq_sendmsg      zmq_send    //  Smooth out 2.x changes
 #   define zmq_recvmsg      zmq_recv
+#   define zmq_ctx_new      zmq_init
+#   define zmq_ctx_term     zmq_term
+#   define zmq_msg_send(m,s,f)  zmq_sendmsg ((s),(m),(f))
+#   define zmq_msg_recv(m,s,f)  zmq_recvmsg ((s),(m),(f))
     //  Older libzmq APIs may be missing some aspects of libzmq v3.0
 #   ifndef ZMQ_ROUTER
 #       define ZMQ_ROUTER       ZMQ_XREP
