@@ -158,6 +158,12 @@ s_self_prepare_udp (self_t *self)
                 zsys_info ("zbeacon: configured, hostname=%s", self->hostname);
         }
     }
+    else
+    {
+        //  No valid interface. Close the socket so that we can try again later
+        zsys_udp_close(self->udpsock);
+        self->udpsock = INVALID_SOCKET;
+    }
 }
 
 
@@ -268,7 +274,8 @@ s_self_handle_udp (self_t *self)
         assert (msg);
         zmsg_addstr (msg, peername);
         zmsg_append (msg, &frame);
-        zmsg_send (&msg, self->pipe);
+        if (zmsg_send (&msg, self->pipe) < 0)
+            zmsg_destroy (&msg);
     }
     else
         zframe_destroy (&frame);
