@@ -1,4 +1,4 @@
-﻿/*  =========================================================================
+/*  =========================================================================
     zhashx - extended generic hash container
 
     Copyright (c) the Contributors as noted in the AUTHORS file.
@@ -24,7 +24,7 @@
 @end
 */
 
-#include "../include/czmq.h"
+#include "czmq_classes.h"
 
 //  Hash table performance parameters
 
@@ -44,8 +44,6 @@ typedef struct _item_t {
     struct _item_t *next;       //  Next item in the hash slot
     size_t index;               //  Index of item in table
     const void *key;            //  Item's original key
-    //  Supporting deprecated v2 functionality; we can't quite replace
-    //  this with strdup/zstr_free as zhashx_insert also uses autofree.
     zhashx_free_fn *free_fn;     //  Value free function if any
 } item_t;
 
@@ -103,20 +101,17 @@ zhashx_t *
 zhashx_new (void)
 {
     zhashx_t *self = (zhashx_t *) zmalloc (sizeof (zhashx_t));
-    if (self) {
-        self->prime_index = INITIAL_PRIME;
-        self->chain_limit = INITIAL_CHAIN;
-        size_t limit = primes [self->prime_index];
-        self->items = (item_t **) zmalloc (sizeof (item_t *) * limit);
-        if (self->items) {
-            self->hasher = s_bernstein_hash;
-            self->key_destructor = (zhashx_destructor_fn *) zstr_free;
-            self->key_duplicator = (zhashx_duplicator_fn *) strdup;
-            self->key_comparator = (zhashx_comparator_fn *) strcmp;
-        }
-        else
-            zhashx_destroy (&self);
-    }
+    assert (self);
+    self->prime_index = INITIAL_PRIME;
+    self->chain_limit = INITIAL_CHAIN;
+    size_t limit = primes [self->prime_index];
+    self->items = (item_t **) zmalloc (sizeof (item_t *) * limit);
+    assert (self->items);
+    self->hasher = s_bernstein_hash;
+    self->key_destructor = (zhashx_destructor_fn *) zstr_free;
+    self->key_duplicator = (zhashx_duplicator_fn *) strdup;
+    self->key_comparator = (zhashx_comparator_fn *) strcmp;
+
     return self;
 }
 
@@ -212,8 +207,7 @@ s_zhashx_rehash (zhashx_t *self, uint new_prime_index)
     size_t limit = primes [self->prime_index];
     size_t new_limit = primes [new_prime_index];
     item_t **new_items = (item_t **) zmalloc (sizeof (item_t *) * new_limit);
-    if (!new_items)
-        return -1;
+    assert (new_items);
 
     //  Move all items to the new hash table, rehashing to
     //  take into account new hash table limit
@@ -260,7 +254,7 @@ zhashx_insert (zhashx_t *self, const void *key, void *value)
             return -1;
         self->chain_limit += CHAIN_GROWS;
     }
-    return s_item_insert (self, key, value) ? 0 : -1;
+    return s_item_insert (self, key, value)? 0: -1;
 }
 
 
@@ -278,8 +272,7 @@ s_item_insert (zhashx_t *self, const void *key, void *value)
     item_t *item = s_item_lookup (self, key);
     if (item == NULL) {
         item = (item_t *) zmalloc (sizeof (item_t));
-        if (!item)
-            return NULL;
+        assert (item);
 
         //  If necessary, take duplicate of item key
         if (self->key_duplicator)
@@ -400,14 +393,11 @@ zhashx_purge (zhashx_t *self)
     if (self->prime_index > INITIAL_PRIME) {
         // Try to shrink hash table
         size_t limit = primes [INITIAL_PRIME];
-        item_t **items =
-            (item_t **) zmalloc (sizeof (item_t *) * limit);
-        if (items) {
-            free (self->items);
-            self->prime_index = INITIAL_PRIME;
-            self->chain_limit = INITIAL_CHAIN;
-            self->items = items;
-        }
+        item_t **items = (item_t **) zmalloc (sizeof (item_t *) * limit);
+        assert (items);
+        self->prime_index = INITIAL_PRIME;
+        self->chain_limit = INITIAL_CHAIN;
+        self->items = items;
     }
 }
 
@@ -474,7 +464,7 @@ zhashx_rename (zhashx_t *self, const void *old_key, const void *new_key)
 //  Returns the item, or NULL if there is no such item.
 
 void *
-zhashx_freefn (zhashx_t *self, const void *key, zhashx_free_fn *free_fn)
+zhashx_freefn (zhashx_t *self, const void *key, zhashx_free_fn free_fn)
 {
     assert (self);
     assert (key);
@@ -534,8 +524,11 @@ zhashx_keys (zhashx_t *self)
 //  a duplicator, then it is used to duplicate all items, and if there
 //  is a destructor then it set as the destructor for the list.
 
-zlistx_t *zhashx_values(zhashx_t *self) {
-    assert(self);
+zlistx_t *
+zhashx_values (zhashx_t *self)
+{
+    assert (self);
+
     zlistx_t *values = zlistx_new ();
     if (!values)
         return NULL;
@@ -555,7 +548,6 @@ zlistx_t *zhashx_values(zhashx_t *self) {
             item = item->next;
         }
      }
-
     return values;
 }
 
@@ -696,7 +688,8 @@ int
 zhashx_load (zhashx_t *self, const char *filename)
 {
     assert (self);
-    zhashx_autofree (self);
+    zhashx_set_destructor (self, (zhashx_destructor_fn *) zstr_free);
+    zhashx_set_duplicator (self, (zhashx_duplicator_fn *) strdup);
 
     //  Whether or not file exists, we'll track the filename and last
     //  modification date (0 for unknown files), so that zhashx_refresh ()
@@ -705,40 +698,32 @@ zhashx_load (zhashx_t *self, const char *filename)
 
     //  Take copy of filename in case self->filename is same string.
     char *filename_copy = strdup (filename);
-    if (filename_copy) {
-        free (self->filename);
-        self->filename = filename_copy;
-        self->modified = zsys_file_modified (self->filename);
-        FILE *handle = fopen (self->filename, "r");
-        if (handle) {
-            char *buffer = (char *) zmalloc (1024);
-            if (buffer) {
-                while (fgets (buffer, 1024, handle)) {
-                    //  Skip lines starting with "#" or that do not look like
-                    //  name=value data.
-                    char *equals = strchr (buffer, '=');
-                    if (buffer [0] == '#' || equals == buffer || !equals)
-                        continue;
+    assert (filename_copy);
+    free (self->filename);
+    self->filename = filename_copy;
+    self->modified = zsys_file_modified (self->filename);
+    FILE *handle = fopen (self->filename, "r");
+    if (handle) {
+        char *buffer = (char *) zmalloc (1024);
+        assert (buffer);
+        while (fgets (buffer, 1024, handle)) {
+            //  Skip lines starting with "#" or that do not look like
+            //  name=value data.
+            char *equals = strchr (buffer, '=');
+            if (buffer [0] == '#' || equals == buffer || !equals)
+                continue;
 
-                    //  Buffer may end in newline, which we don't want
-                    if (buffer [strlen (buffer) - 1] == '\n')
-                        buffer [strlen (buffer) - 1] = 0;
-                    *equals++ = 0;
-                    zhashx_update (self, buffer, equals);
-                }
-                free (buffer);
-            }
-            else {
-                fclose (handle);
-                return -1; //  Out of memory
-            }
-            fclose (handle);
+            //  Buffer may end in newline, which we don't want
+            if (buffer [strlen (buffer) - 1] == '\n')
+                buffer [strlen (buffer) - 1] = 0;
+            *equals++ = 0;
+            zhashx_update (self, buffer, equals);
         }
-        else
-            return -1; //  Failed to open file for reading
+        free (buffer);
+        fclose (handle);
     }
     else
-        return -1; //  Out of memory
+        return -1; //  Failed to open file for reading
 
     return 0;
 }
@@ -777,6 +762,78 @@ zhashx_refresh (zhashx_t *self)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Same as pack but uses a user-defined serializer function to convert items
+//  into longstr.
+//  Caller owns return value and must destroy it when done.
+
+zframe_t *
+zhashx_pack_own (zhashx_t *self, zhashx_serializer_fn serializer)
+{
+    assert (self);
+
+    //  First, calculate packed data size
+    size_t frame_size = 4;      //  Dictionary size, number-4
+    uint index;
+    uint vindex = 0;
+    size_t limit = primes [self->prime_index];
+    char **values = (char **) zmalloc (self->size * sizeof (char*));
+    for (index = 0; index < limit; index++) {
+        item_t *item = self->items [index];
+        while (item) {
+            //  We store key as short string
+            frame_size += 1 + strlen ((char *) item->key);
+            //  We store value as long string
+            if (serializer)
+                values [vindex] = serializer (item->value);
+            else
+                values [vindex] = (char *) item->value;
+
+            frame_size += 4 + strlen ((char *) values [vindex]);
+            item = item->next;
+            vindex++;
+        }
+    }
+    //  Now serialize items into the frame
+    zframe_t *frame = zframe_new (NULL, frame_size);
+    if (!frame) {
+        free (values);
+        return NULL;
+    }
+
+    byte *needle = zframe_data (frame);
+    //  Store size as number-4
+    *(uint32_t *) needle = htonl ((u_long) self->size);
+    needle += 4;
+    vindex = 0;
+    for (index = 0; index < limit; index++) {
+        item_t *item = self->items [index];
+        while (item) {
+            //  Store key as string
+            *needle++ = (byte) strlen ((char *) item->key);
+            memcpy (needle, item->key, strlen ((char *) item->key));
+            needle += strlen ((char *) item->key);
+
+            //  Store value as longstr
+            size_t lenth = strlen (values [vindex]);
+            *(uint32_t *) needle = htonl ((u_long) lenth);
+            needle += 4;
+            memcpy (needle, (char *) values [vindex], strlen ((char *) values [vindex]));
+            needle += strlen ((char *) values [vindex]);
+            item = item->next;
+
+            //  Destroy serialized value
+            if (serializer)
+                zstr_free (&values [vindex]);
+
+            vindex++;
+        }
+    }
+    free (values);
+    return frame;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Serialize hash table to a binary frame that can be sent in a message.
 //  The packed format is compatible with the 'dictionary' type defined in
@@ -802,62 +859,21 @@ zhashx_refresh (zhashx_t *self)
 zframe_t *
 zhashx_pack (zhashx_t *self)
 {
-    assert (self);
-
-    //  First, calculate packed data size
-    size_t frame_size = 4;      //  Dictionary size, number-4
-    uint index;
-    size_t limit = primes [self->prime_index];
-    for (index = 0; index < limit; index++) {
-        item_t *item = self->items [index];
-        while (item) {
-            //  We store key as short string
-            frame_size += 1 + strlen ((char *) item->key);
-            //  We store value as long string
-            frame_size += 4 + strlen ((char *) item->value);
-            item = item->next;
-        }
-    }
-    //  Now serialize items into the frame
-    zframe_t *frame = zframe_new (NULL, frame_size);
-    if (!frame)
-        return NULL;
-    byte *needle = zframe_data (frame);
-    //  Store size as number-4
-    *(uint32_t *) needle = htonl ((u_long) self->size);
-    needle += 4;
-    for (index = 0; index < limit; index++) {
-        item_t *item = self->items [index];
-        while (item) {
-            //  Store key as string
-            *needle++ = (byte) strlen ((char *) item->key);
-            memcpy (needle, item->key, strlen ((char *) item->key));
-            needle += strlen ((char *) item->key);
-
-            //  Store value as longstr
-            size_t lenth = strlen ((char *) item->value);
-            *(uint32_t *) needle = htonl ((u_long) lenth);
-            needle += 4;
-            memcpy (needle, (char *) item->value, strlen ((char *) item->value));
-            needle += strlen ((char *) item->value);
-            item = item->next;
-        }
-    }
-    return frame;
+    return zhashx_pack_own (self, NULL);
 }
 
 
 //  --------------------------------------------------------------------------
-//  Unpack binary frame into a new hash table. Packed data must follow format
-//  defined by zhashx_pack. Hash table is set to autofree. An empty frame
-//  unpacks to an empty hash table.
+//  Same as unpack but uses a user-defined deserializer function to convert
+//  a longstr back into item format.
 
 zhashx_t *
-zhashx_unpack (zframe_t *frame)
+zhashx_unpack_own (zframe_t *frame, zhashx_deserializer_fn deserializer)
 {
     zhashx_t *self = zhashx_new ();
     if (!self)
         return NULL;
+
     assert (frame);
     if (zframe_size (frame) < 4)
         return self;            //  Arguable...
@@ -882,16 +898,22 @@ zhashx_unpack (zframe_t *frame)
                 //  Be wary of malformed frames
                 if (needle + value_size <= ceiling) {
                     char *value = (char *) zmalloc (value_size + 1);
-                    if (!value) {
-                        zhashx_destroy (&self);
-                        return NULL;
-                    }
+                    assert (value);
                     memcpy (value, needle, value_size);
                     value [value_size] = 0;
                     needle += value_size;
 
-                    //  Hash takes ownership of value
-                    if (zhashx_insert (self, key, value)) {
+                    //  Convert string to real value
+                    void *real_value;
+                    if (deserializer) {
+                        real_value = deserializer (value);
+                        zstr_free (&value);
+                    }
+                    else
+                        real_value = value;
+
+                    //  Hash takes ownership of real_value
+                    if (zhashx_insert (self, key, real_value)) {
                         zhashx_destroy (&self);
                         break;
                     }
@@ -900,12 +922,27 @@ zhashx_unpack (zframe_t *frame)
         }
     }
     //  Hash will free values in destructor
-    if (self)
-        zhashx_autofree (self);
+    if (self) {
+        zhashx_set_destructor (self, (zhashx_destructor_fn *) zstr_free);
+        zhashx_set_duplicator (self, (zhashx_duplicator_fn *) strdup);
+    }
+
     return self;
 }
 
 
+//  --------------------------------------------------------------------------
+//  Unpack binary frame into a new hash table. Packed data must follow format
+//  defined by zhashx_pack. Hash table is set to autofree. An empty frame
+//  unpacks to an empty hash table.
+
+zhashx_t *
+zhashx_unpack (zframe_t *frame)
+{
+    return zhashx_unpack_own (frame, NULL);
+}
+
+
 //  --------------------------------------------------------------------------
 //  Make a copy of the list; items are duplicated if you set a duplicator
 //  for the list, otherwise not. Copying a null reference returns a null
@@ -1028,7 +1065,8 @@ zhashx_dup_v2 (zhashx_t *self)
 
     zhashx_t *copy = zhashx_new ();
     if (copy) {
-        zhashx_autofree (copy);
+        zhashx_set_destructor (copy, (zhashx_destructor_fn *) zstr_free);
+        zhashx_set_duplicator (copy, (zhashx_duplicator_fn *) strdup);
         uint index;
         size_t limit = primes [self->prime_index];
         for (index = 0; index < limit; index++) {
@@ -1046,53 +1084,38 @@ zhashx_dup_v2 (zhashx_t *self)
 }
 
 
-//  --------------------------------------------------------------------------
-//  DEPRECATED as clumsy -- use set_destructor instead
-//  Set hash for automatic value destruction
-
-void
-zhashx_autofree (zhashx_t *self)
-{
-    assert (self);
-    zhashx_set_destructor (self, (zhashx_destructor_fn *) zstr_free);
-    zhashx_set_duplicator (self, (zhashx_duplicator_fn *) strdup);
-}
-
-
-//  --------------------------------------------------------------------------
-//  DEPRECATED as clumsy -- use zhashx_first/_next instead
-//  Apply function to each item in the hash table. Items are iterated in no
-//  defined order.  Stops if callback function returns non-zero and returns
-//  final return code from callback function (zero = success).
-
-int
-zhashx_foreach (zhashx_t *self, zhashx_foreach_fn *callback, void *argument)
-{
-    assert (self);
-
-    uint index;
-    size_t limit = primes [self->prime_index];
-    for (index = 0; index < limit; index++) {
-        item_t *item = self->items [index];
-        while (item) {
-            //  Invoke callback, passing item properties and argument
-            item_t *next = item->next;
-            int rc = callback ((const char *) item->key, item->value, argument);
-            if (rc)
-                return rc;      //  End if non-zero return code
-            item = next;
-        }
-    }
-    return 0;
-}
-
-
 //  --------------------------------------------------------------------------
 //  Runs selftest of class
 //
 
+#ifdef CZMQ_BUILD_DRAFT_API
+char *
+s_test_serialize_int (const void *item)
+{
+    int *int_item = (int *) item;
+    char *str_item = (char *) zmalloc (sizeof (char) * 10);
+    sprintf (str_item, "%d", *int_item);
+    return str_item;
+}
+
+void *
+s_test_deserialze_int (const char *str_item)
+{
+    int *int_item = (int *) zmalloc (sizeof (int));
+    sscanf (str_item, "%d", int_item);
+    return int_item;
+}
+
 void
-zhashx_test (int verbose)
+s_test_destroy_int (void **item)
+{
+    int *int_item = (int *) *item;
+    free (int_item);
+}
+#endif // CZMQ_BUILD_DRAFT_API
+
+void
+zhashx_test (bool verbose)
 {
     printf (" * zhashx: ");
 
@@ -1191,6 +1214,28 @@ zhashx_test (int verbose)
     assert (streq (item, "dead beef"));
     zhashx_destroy (&copy);
 
+#ifdef CZMQ_BUILD_DRAFT_API
+    //  Test own pack/unpack methods
+    zhashx_t *own_hash = zhashx_new ();
+    zhashx_set_destructor (own_hash, s_test_destroy_int);
+    assert (own_hash);
+    int *val1 = (int *) zmalloc (sizeof (int));
+    int *val2 = (int *) zmalloc (sizeof (int));
+    *val1 = 25;
+    *val2 = 100;
+    zhashx_insert (own_hash, "val1", val1);
+    zhashx_insert (own_hash, "val2", val2);
+    frame = zhashx_pack_own (own_hash, s_test_serialize_int);
+    copy = zhashx_unpack_own (frame, s_test_deserialze_int);
+    zhashx_set_destructor (copy, s_test_destroy_int);
+    zframe_destroy (&frame);
+    assert (zhashx_size (copy) == 2);
+    assert (*((int *) zhashx_lookup (copy, "val1")) == 25);
+    assert (*((int *) zhashx_lookup (copy, "val2")) == 100);
+    zhashx_destroy (&copy);
+    zhashx_destroy (&own_hash);
+#endif // CZMQ_BUILD_DRAFT_API
+
     //  Test save and load
     zhashx_comment (hash, "This is a test file");
     zhashx_comment (hash, "Created by %s", "czmq_selftest");
@@ -1242,10 +1287,11 @@ zhashx_test (int verbose)
     zhashx_destroy (&hash);
     assert (hash == NULL);
 
-    //  Test autofree; automatically copies and frees string values
+    //  Test destructor; automatically copies and frees string values
     hash = zhashx_new ();
     assert (hash);
-    zhashx_autofree (hash);
+    zhashx_set_destructor (hash, (zhashx_destructor_fn *) zstr_free);
+    zhashx_set_duplicator (hash, (zhashx_duplicator_fn *) strdup);
     char value [255];
     strcpy (value, "This is a string");
     rc = zhashx_insert (hash, "key1", value);
