@@ -49,6 +49,26 @@ struct _zsock_t {
     uint32_t routing_id;        //  Routing ID for server sockets
 };
 
+#ifndef CZMQ_BUILD_DRAFT_API
+CZMQ_PRIVATE zsock_t *
+    zsock_new_server_checked (const char *endpoint, const char *filename, size_t line_nbr);
+
+CZMQ_PRIVATE zsock_t *
+    zsock_new_client_checked (const char *endpoint, const char *filename, size_t line_nbr);
+
+CZMQ_PRIVATE zsock_t *
+    zsock_new_radio_checked (const char *endpoint, const char *filename, size_t line_nbr);
+
+CZMQ_PRIVATE zsock_t *
+    zsock_new_dish_checked (const char *endpoint, const char *filename, size_t line_nbr);
+
+CZMQ_PRIVATE zsock_t *
+    zsock_new_gather_checked (const char *endpoint, const char *filename, size_t line_nbr);
+
+CZMQ_PRIVATE zsock_t *
+    zsock_new_scatter_checked (const char *endpoint, const char *filename, size_t line_nbr);
+#endif // CZMQ_BUILD_DRAFT_API
+
 
 //  --------------------------------------------------------------------------
 //  Create a new socket. This macro passes the caller source and line
@@ -1650,7 +1670,12 @@ zsock_signal (void *self, byte status)
 {
     assert (self);
     zmsg_t *msg = zmsg_new_signal (status);
-    return zmsg_send (&msg, self);
+    if (zmsg_send (&msg, self) < 0) {
+        zmsg_destroy (&msg);
+        return -1;
+    }
+    else
+        return 0;
 }
 
 
@@ -1906,9 +1931,12 @@ zsock_test (bool verbose)
     //  Test zsock_send/recv pictures
     uint8_t  number1 = 123;
     uint16_t number2 = 123 * 123;
-    uint32_t number4 = 123 * 123 * 123;
-    uint64_t number4_MAX = UINT32_MAX;
-    uint64_t number8 = 123 * 123 * 123 * 123;
+    uint32_t number4 = 123 * 123;
+    number4 *= 123;
+    uint32_t number4_MAX = UINT32_MAX;
+    uint64_t number8 = 123 * 123;
+    number8 *= 123;
+    number8 *= 123;
     uint64_t number8_MAX = UINT64_MAX;
 
     zchunk_t *chunk = zchunk_new ("HELLO", 5);
@@ -1941,7 +1969,7 @@ zsock_test (bool verbose)
     byte *data;
     size_t size;
     char *pointer;
-    number8_MAX = number8 = number4 = number2 = number1 = 0;
+    number8_MAX = number8 = number4_MAX = number4 = number2 = number1 = 0ULL;
     rc = zsock_recv (reader, "i124488zsbcfUhp",
                      &integer, &number1, &number2, &number4, &number4_MAX,
                      &number8, &number8_MAX, &string, &data, &size, &chunk,
