@@ -1,4 +1,4 @@
-﻿/*  =========================================================================
+/*  =========================================================================
     zsys - system-level methods
 
     Copyright (c) the Contributors as noted in the AUTHORS file.
@@ -20,8 +20,7 @@
 @end
 */
 
-#include "platform.h"
-#include "../include/czmq.h"
+#include "czmq_classes.h"
 
 //  --------------------------------------------------------------------------
 //  Signal handling
@@ -29,12 +28,12 @@
 //  These are global variables accessible to CZMQ application code
 volatile int zsys_interrupted = 0;  //  Current name
 volatile int zctx_interrupted = 0;  //  Deprecated name
-volatile uint64_t zsys_allocs = 0;
 
 static void s_signal_handler (int signal_value);
 
 //  We use these variables for signal handling
 static bool s_first_time = true;
+static bool handle_signals = true;
 #if defined (__UNIX__)
 static struct sigaction sigint_default;
 static struct sigaction sigterm_default;
@@ -65,12 +64,16 @@ static bool s_initialized = false;
 //  from the environment, or via the zsys_set_xxx API.
 static size_t s_io_threads = 1;     //  ZSYS_IO_THREADS=1
 static size_t s_max_sockets = 1024; //  ZSYS_MAX_SOCKETS=1024
+static int s_max_msgsz = INT_MAX;   //  ZSYS_MAX_MSGSZ=INT_MAX
 static size_t s_linger = 0;         //  ZSYS_LINGER=0
 static size_t s_sndhwm = 1000;      //  ZSYS_SNDHWM=1000
 static size_t s_rcvhwm = 1000;      //  ZSYS_RCVHWM=1000
 static size_t s_pipehwm = 1000;     //  ZSYS_PIPEHWM=1000
 static int s_ipv6 = 0;              //  ZSYS_IPV6=0
 static char *s_interface = NULL;    //  ZSYS_INTERFACE=
+static char *s_ipv6_address = NULL; //  ZSYS_IPV6_ADDRESS=
+static char *s_ipv6_mcast_address = NULL; //  ZSYS_IPV6_MCAST_ADDRESS=
+static int s_auto_use_fd = 0;       //  ZSYS_AUTO_USE_FD=0
 static char *s_logident = NULL;     //  ZSYS_LOGIDENT=
 static FILE *s_logstream = NULL;    //  ZSYS_LOGSTREAM=stdout/stderr
 static bool s_logsystem = false;    //  ZSYS_LOGSYSTEM=true/false
@@ -82,7 +85,7 @@ static size_t s_open_sockets = 0;
 //  We keep a list of open sockets to report leaks to developers
 static zlist_t *s_sockref_list = NULL;
 
-//  This defines a single zsocket_new() caller instance
+//  This defines a single zsock_new() caller instance
 typedef struct {
     void *handle;
     int type;
@@ -130,6 +133,9 @@ zsys_init (void)
     if (getenv ("ZSYS_MAX_SOCKETS"))
         s_max_sockets = atoi (getenv ("ZSYS_MAX_SOCKETS"));
 
+    if (getenv ("ZSYS_MAX_MSGSZ"))
+        s_max_msgsz = atoi (getenv ("ZSYS_MAX_MSGSZ"));
+
     if (getenv ("ZSYS_LINGER"))
         s_linger = atoi (getenv ("ZSYS_LINGER"));
 
@@ -162,10 +168,11 @@ zsys_init (void)
         if (streq (getenv ("ZSYS_LOGSYSTEM"), "false"))
             s_logsystem = false;
     }
-    //  Catch SIGINT and SIGTERM unless ZSYS_SIGHANDLER=false
-    if ((getenv ("ZSYS_SIGHANDLER") == NULL
-	 ||  strneq (getenv ("ZSYS_SIGHANDLER"), "false")) && s_first_time)
-        zsys_catch_interrupts ();
+
+    if (getenv ("ZSYS_AUTO_USE_FD"))
+        s_auto_use_fd = atoi (getenv ("ZSYS_AUTO_USE_FD"));
+
+    zsys_catch_interrupts ();
 
     ZMUTEX_INIT (s_mutex);
     s_sockref_list = zlist_new ();
@@ -179,9 +186,7 @@ zsys_init (void)
     assert (!s_process_ctx);
     //  We use zmq_init/zmq_term to keep compatibility back to ZMQ v2
     s_process_ctx = zmq_init ((int) s_io_threads);
-#if (ZMQ_VERSION >= ZMQ_MAKE_VERSION (3, 2, 0))
-    //  TODO: this causes TravisCI to break; libzmq does not return a
-    //  valid socket on zmq_socket(), after this...
+#if defined (ZMQ_MAX_SOCKETS)
     zmq_ctx_set (s_process_ctx, ZMQ_MAX_SOCKETS, (int) s_max_sockets);
 #endif
     s_initialized = true;
@@ -191,12 +196,20 @@ zsys_init (void)
     if (getenv ("ZSYS_INTERFACE"))
         zsys_set_interface (getenv ("ZSYS_INTERFACE"));
 
+    if (getenv ("ZSYS_IPV6_ADDRESS"))
+        zsys_set_ipv6_address (getenv ("ZSYS_IPV6_ADDRESS"));
+
+    if (getenv ("ZSYS_IPV6_MCAST_ADDRESS"))
+        zsys_set_ipv6_mcast_address (getenv ("ZSYS_IPV6_MCAST_ADDRESS"));
+
     if (getenv ("ZSYS_LOGIDENT"))
         zsys_set_logident (getenv ("ZSYS_LOGIDENT"));
 
     if (getenv ("ZSYS_LOGSENDER"))
         zsys_set_logsender (getenv ("ZSYS_LOGSENDER"));
 
+    zsys_set_max_msgsz (s_max_msgsz);
+
     return s_process_ctx;
 }
 
@@ -222,9 +235,8 @@ zsys_shutdown (void)
         zclock_sleep (200);
 
     //  Close logsender socket if opened (don't do this in critical section)
-    if (s_logsender) {
+    if (s_logsender) 
         zsock_destroy (&s_logsender);
-    }
 
     //  No matter, we are now going to shut down
     //  Print the source reference for any sockets the app did not
@@ -252,6 +264,8 @@ zsys_shutdown (void)
 
     //  Free dynamically allocated properties
     free (s_interface);
+    free (s_ipv6_address);
+    free (s_ipv6_mcast_address);
     free (s_logident);
 
 #if defined (__UNIX__)
@@ -283,8 +297,14 @@ zsys_socket (int type, const char *filename, size_t line_nbr)
         //  Configure socket with process defaults
         zsock_set_linger (handle, (int) s_linger);
 #if (ZMQ_VERSION_MAJOR == 2)
-        //  For ZeroMQ/2.x we use sndhwm for both send and receive
-        zsock_set_hwm (handle, s_sndhwm);
+        // TODO: v2/v3 socket api in zsock_option.inc are not public (not
+        // added to include/zsock.h) so we have to use zmq_setsockopt directly
+        // This should be fixed and zsock_set_hwm should be used instead
+#       if defined (ZMQ_HWM)
+        uint64_t value = s_sndhwm;
+        int rc = zmq_setsockopt (handle, ZMQ_HWM, &value, sizeof (uint64_t));
+        assert (rc == 0 || zmq_errno () == ETERM);
+#       endif
 #else
         //  For later versions we use separate SNDHWM and RCVHWM
         zsock_set_sndhwm (handle, (int) s_sndhwm);
@@ -355,11 +375,20 @@ zsys_sockname (int socktype)
     char *type_names [] = {
         "PAIR", "PUB", "SUB", "REQ", "REP",
         "DEALER", "ROUTER", "PULL", "PUSH",
-        "XPUB", "XSUB", "STREAM"
+        "XPUB", "XSUB", "STREAM",
+        "SERVER", "CLIENT",
+        "RADIO", "DISH",
+        "SCATTER", "GATHER"
     };
     //  This array matches ZMQ_XXX type definitions
     assert (ZMQ_PAIR == 0);
-#if defined (ZMQ_STREAM)
+#if defined (ZMQ_SCATTER)
+    assert (socktype >= 0 && socktype <= ZMQ_SCATTER);
+#elif defined (ZMQ_DISH)
+    assert (socktype >= 0 && socktype <= ZMQ_DISH);
+#elif defined (ZMQ_CLIENT)
+    assert (socktype >= 0 && socktype <= ZMQ_CLIENT);
+#elif defined (ZMQ_STREAM)
     assert (socktype >= 0 && socktype <= ZMQ_STREAM);
 #else
     assert (socktype >= 0 && socktype <= ZMQ_XSUB);
@@ -378,14 +407,23 @@ zsys_create_pipe (zsock_t **backend_p)
 {
     zsock_t *frontend = zsock_new (ZMQ_PAIR);
     zsock_t *backend = zsock_new (ZMQ_PAIR);
-    if (!frontend || !backend) {
-        zsock_destroy (&frontend);
-        zsock_destroy (&backend);
-        return frontend;
-    }
+    assert (frontend);
+    assert (backend);
+
 #if (ZMQ_VERSION_MAJOR == 2)
-    zsock_set_hwm (frontend, zsys_pipehwm ());
-    zsock_set_hwm (backend, zsys_pipehwm ());
+    // TODO: v2/v3 socket api in zsock_option.inc are not public (not
+    // added to include/zsock.h) so we have to use zmq_setsockopt directly
+    // This should be fixed and zsock_set_hwm should be used instead
+#   if defined (ZMQ_HWM)
+    uint64_t value = zsys_pipehwm ();
+    int ret = zmq_setsockopt (zsock_resolve (frontend), ZMQ_HWM, &value,
+        sizeof (uint64_t));
+    assert (ret == 0 || zmq_errno () == ETERM);
+    value = zsys_pipehwm ();
+    ret = zmq_setsockopt (zsock_resolve (backend), ZMQ_HWM, &value,
+        sizeof (uint64_t));
+    assert (ret == 0 || zmq_errno () == ETERM);
+#   endif
 #else
     zsock_set_sndhwm (frontend, (int) zsys_pipehwm ());
     zsock_set_sndhwm (backend, (int) zsys_pipehwm ());
@@ -419,9 +457,10 @@ zsys_handler_set (zsys_handler_fn *handler_fn)
     if (!handler_fn) {
         //  Disable existing or future signal handling
         zsys_handler_reset ();
-        s_first_time = false;
+        handle_signals = false;
     }
     else {
+        handle_signals = true;
 #if defined (__UNIX__)
         if (s_first_time) {
             //  If first time, save default handlers
@@ -458,7 +497,7 @@ zsys_handler_reset (void)
 {
 #if defined (__UNIX__)
     //  Restore default handlers if not already done
-    if (!s_first_time) {
+    if (handle_signals && !s_first_time) {
         sigaction (SIGINT, &sigint_default, NULL);
         sigaction (SIGTERM, &sigterm_default, NULL);
         sigint_default.sa_handler = NULL;
@@ -466,7 +505,7 @@ zsys_handler_reset (void)
         s_first_time = true;
     }
 #elif defined (__WINDOWS__)
-    if (!s_first_time) {
+    if (handle_signals && !s_first_time) {
         SetConsoleCtrlHandler (s_handler_fn_shim, FALSE);
         installed_handler_fn = NULL;
         s_first_time = true;
@@ -482,7 +521,11 @@ zsys_handler_reset (void)
 void
 zsys_catch_interrupts (void)
 {
-    zsys_handler_set (s_signal_handler);
+    //  Catch SIGINT and SIGTERM unless ZSYS_SIGHANDLER=false
+    if ((getenv ("ZSYS_SIGHANDLER") == NULL
+    ||   strneq (getenv ("ZSYS_SIGHANDLER"), "false"))
+    &&   handle_signals)
+        zsys_handler_set (s_signal_handler);
 }
 
 //  Default internal signal handler
@@ -648,7 +691,7 @@ zsys_dir_create (const char *pathname, ...)
         *slash = '/';
         slash = strchr (slash + 1, '/');
     }
-    free (formatted);
+    zstr_free (&formatted);
     return 0;
 }
 
@@ -671,7 +714,7 @@ zsys_dir_delete (const char *pathname, ...)
 #else
     int rc = rmdir (formatted);
 #endif
-    free (formatted);
+    zstr_free (&formatted);
     return rc;
 }
 
@@ -812,7 +855,11 @@ zsys_udp_new (bool routable)
 {
     //  We haven't implemented multicast yet
     assert (!routable);
-    SOCKET udpsock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+    SOCKET udpsock;
+    if (zsys_ipv6 ())
+        udpsock = socket (AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
+    else
+        udpsock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
     if (udpsock == INVALID_SOCKET) {
         zsys_socket_error ("socket");
         return INVALID_SOCKET;
@@ -858,7 +905,7 @@ zsys_udp_close (SOCKET handle)
 //  interface having disappeared (happens easily with WiFi)
 
 int
-zsys_udp_send (SOCKET udpsock, zframe_t *frame, inaddr_t *address)
+zsys_udp_send (SOCKET udpsock, zframe_t *frame, inaddr_t *address, int addrlen)
 {
     assert (frame);
     assert (address);
@@ -866,7 +913,7 @@ zsys_udp_send (SOCKET udpsock, zframe_t *frame, inaddr_t *address)
     if (sendto (udpsock,
         (char *) zframe_data (frame), (int) zframe_size (frame),
         0, //  Flags
-        (struct sockaddr *) address, (int) sizeof (inaddr_t)) == -1) {
+        (struct sockaddr *) address, addrlen) == -1) {
         zsys_debug ("zsys_udp_send: failed, reason=%s", strerror (errno));
         return -1;              //  UDP broadcast not possible
     }
@@ -880,26 +927,32 @@ zsys_udp_send (SOCKET udpsock, zframe_t *frame, inaddr_t *address)
 //  The peername must be a char [INET_ADDRSTRLEN] array.
 
 zframe_t *
-zsys_udp_recv (SOCKET udpsock, char *peername)
+zsys_udp_recv (SOCKET udpsock, char *peername, int peerlen)
 {
     char buffer [UDP_FRAME_MAX];
-    inaddr_t address;
-    socklen_t address_len = sizeof (inaddr_t);
+    in6addr_t address6;
+#if (!defined (__WINDOWS__))
+    inaddr_t *address = (inaddr_t *) &address6;
+#endif
+    socklen_t address_len = sizeof (in6addr_t);
     ssize_t size = recvfrom (
         udpsock,
         buffer, UDP_FRAME_MAX,
         0,      //  Flags
-        (struct sockaddr *) &address, &address_len);
+        (struct sockaddr *) &address6, &address_len);
 
     if (size == SOCKET_ERROR)
         zsys_socket_error ("recvfrom");
 
     //  Get sender address as printable string
 #if (defined (__WINDOWS__))
-    getnameinfo ((struct sockaddr *) &address, address_len,
-                 peername, INET_ADDRSTRLEN, NULL, 0, NI_NUMERICHOST);
+    getnameinfo ((struct sockaddr *) &address6, address_len,
+                 peername, peerlen, NULL, 0, NI_NUMERICHOST);
 #else
-    inet_ntop (AF_INET, &address.sin_addr, peername, address_len);
+    if (address6.sin6_family == AF_INET6)
+        inet_ntop (AF_INET6, &address6.sin6_addr, peername, address_len);
+    else
+        inet_ntop (AF_INET, &address->sin_addr, peername, address_len);
 #endif
     return zframe_new (buffer, size);
 }
@@ -912,6 +965,8 @@ zsys_udp_recv (SOCKET udpsock, char *peername)
 void
 zsys_socket_error (const char *reason)
 {
+    bool check_errno;
+
 #if defined (__WINDOWS__)
     switch (WSAGetLastError ()) {
         case WSAEINTR:        errno = EINTR;      break;
@@ -923,35 +978,39 @@ zsys_socket_error (const char *reason)
         case WSAECONNABORTED: errno = EPIPE;      break;
         case WSAESHUTDOWN:    errno = ECONNRESET; break;
         case WSAEINVAL:       errno = EPIPE;      break;
+        case WSAEADDRNOTAVAIL: errno = EADDRNOTAVAIL; break;
+        case WSAEADDRINUSE:   errno = EADDRINUSE; break;
         default:              errno = GetLastError ();
     }
 #endif
-    if (errno == EAGAIN
-    ||  errno == ENETDOWN
-    ||  errno == EHOSTUNREACH
-    ||  errno == ENETUNREACH
-    ||  errno == EINTR
-    ||  errno == EPIPE
-    ||  errno == ECONNRESET
+
+    check_errno = (errno == EAGAIN
+                  ||  errno == ENETDOWN
+                  ||  errno == EHOSTUNREACH
+                  ||  errno == ENETUNREACH
+                  ||  errno == EINTR
+                  ||  errno == EPIPE
+                  ||  errno == ECONNRESET);
 #if defined (ENOPROTOOPT)
-    ||  errno == ENOPROTOOPT
+    check_errno = (check_errno ||  errno == ENOPROTOOPT);
 #endif
 #if defined (EHOSTDOWN)
-    ||  errno == EHOSTDOWN
+    check_errno = (check_errno ||  errno == EHOSTDOWN);
 #endif
 #if defined (EOPNOTSUPP)
-    ||  errno == EOPNOTSUPP
+    check_errno = (check_errno ||  errno == EOPNOTSUPP);
 #endif
 #if defined (EWOULDBLOCK)
-    ||  errno == EWOULDBLOCK
+    check_errno = (check_errno ||  errno == EWOULDBLOCK);
 #endif
 #if defined (EPROTO)
-    ||  errno == EPROTO
+    check_errno = (check_errno ||  errno == EPROTO);
 #endif
 #if defined (ENONET)
-    ||  errno == ENONET
+    check_errno = (check_errno ||  errno == ENONET);
 #endif
-    )
+
+    if (check_errno)
         return;             //  Ignore error and try again
     else {
         zsys_error ("(UDP) error '%s' on %s", strerror (errno), reason);
@@ -1072,8 +1131,8 @@ zsys_run_as (const char *lockfile, const char *group, const char *user)
         }
         //   We record the current process id in the lock file
         char pid_buffer [10];
-        snprintf (pid_buffer, sizeof (pid_buffer), "%6d\n", getpid ());
-        if (write (handle, pid_buffer, strlen (pid_buffer)) != strlen (pid_buffer)) {
+        snprintf (pid_buffer, sizeof (pid_buffer), "%6" PRIi64 "\n", (int64_t)getpid ());
+        if ((size_t) write (handle, pid_buffer, strlen (pid_buffer)) != strlen (pid_buffer)) {
             zsys_error ("cannot write to lockfile: %s", strerror (errno));
             close (handle);
             return -1;
@@ -1162,12 +1221,11 @@ zsys_set_io_threads (size_t io_threads)
     zmq_term (s_process_ctx);
     s_io_threads = io_threads;
     s_process_ctx = zmq_init ((int) s_io_threads);
-#if (ZMQ_VERSION >= ZMQ_MAKE_VERSION (3, 2, 0))
-    //  TODO: this causes TravisCI to break; libzmq does not return a
-    //  valid socket on zmq_socket(), after this...
+#if defined (ZMQ_MAX_SOCKETS)
     zmq_ctx_set (s_process_ctx, ZMQ_MAX_SOCKETS, (int) s_max_sockets);
 #endif
     ZMUTEX_UNLOCK (s_mutex);
+    zsys_set_max_msgsz (s_max_msgsz);
 }
 
 
@@ -1187,6 +1245,9 @@ zsys_set_max_sockets (size_t max_sockets)
         zsys_error ("zsys_max_sockets() is not valid after creating sockets");
     assert (s_open_sockets == 0);
     s_max_sockets = max_sockets? max_sockets: zsys_socket_limit ();
+#if defined (ZMQ_MAX_SOCKETS)
+    zmq_ctx_set (s_process_ctx, ZMQ_MAX_SOCKETS, (int) s_max_sockets);
+#endif
     ZMUTEX_UNLOCK (s_mutex);
 }
 
@@ -1198,7 +1259,7 @@ size_t
 zsys_socket_limit (void)
 {
     size_t socket_limit;
-#if (ZMQ_VERSION >= ZMQ_MAKE_VERSION (4, 1, 0))
+#if defined (ZMQ_SOCKET_LIMIT)
     if (s_process_ctx)
         socket_limit = (size_t) zmq_ctx_get (s_process_ctx, ZMQ_SOCKET_LIMIT);
     else {
@@ -1218,6 +1279,42 @@ zsys_socket_limit (void)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Configure the maximum allowed size of a message sent.
+//  The default is INT_MAX.
+
+void
+zsys_set_max_msgsz (int max_msgsz)
+{
+    if (max_msgsz < 0)
+        return;
+
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+    s_max_msgsz = max_msgsz;
+#if defined (ZMQ_MAX_MSGSZ)
+    zmq_ctx_set (s_process_ctx, ZMQ_MAX_MSGSZ, (int) s_max_msgsz);
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Return maximum message size.
+
+int
+zsys_max_msgsz (void)
+{
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+#if defined (ZMQ_MAX_MSGSZ)
+    s_max_msgsz = zmq_ctx_get (s_process_ctx, ZMQ_MAX_MSGSZ);
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+    return s_max_msgsz;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Configure the default linger timeout in msecs for new zsock instances.
 //  You can also set this separately on each zsock_t instance. The default
@@ -1314,6 +1411,16 @@ zsys_set_ipv6 (int ipv6)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Return use of IPv6 for zsock instances.
+
+int
+zsys_ipv6 (void)
+{
+    return s_ipv6;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Set network interface name to use for broadcasts, particularly zbeacon.
 //  This lets the interface be configured for test environments where required.
@@ -1342,6 +1449,88 @@ zsys_interface (void)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Set IPv6 address to use zbeacon socket, particularly for receiving zbeacon.
+//  This needs to be set IPv6 is enabled as IPv6 can have multiple addresses
+//  on a given interface. If the environment variable ZSYS_IPV6_ADDRESS is set,
+//  use that as the default IPv6 address.
+
+void
+zsys_set_ipv6_address (const char *value)
+{
+    zsys_init ();
+    free (s_ipv6_address);
+    s_ipv6_address = strdup (value);
+    assert (s_ipv6_address);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Return IPv6 address to use for zbeacon reception, or "" if none was set.
+
+const char *
+zsys_ipv6_address (void)
+{
+    return s_ipv6_address? s_ipv6_address: "";
+}
+
+
+//  --------------------------------------------------------------------------
+//  Set IPv6 milticast address to use for sending zbeacon messages. This needs
+//  to be set if IPv6 is enabled. If the environment variable
+//  ZSYS_IPV6_MCAST_ADDRESS is set, use that as the default IPv6 multicast
+//  address.
+
+void
+zsys_set_ipv6_mcast_address (const char *value)
+{
+    zsys_init ();
+    free (s_ipv6_mcast_address);
+    s_ipv6_mcast_address = strdup (value);
+    assert (s_ipv6_mcast_address);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Return IPv6 multicast address to use for sending zbeacon, or "" if none was
+//  set.
+
+const char *
+zsys_ipv6_mcast_address (void)
+{
+    return s_ipv6_mcast_address? s_ipv6_mcast_address: "";
+}
+
+
+//  --------------------------------------------------------------------------
+//  Configure the automatic use of pre-allocated FDs when creating new sockets.
+//  If 0 (default), nothing will happen. Else, when a new socket is bound, the
+//  system API will be used to check if an existing pre-allocated FD with a
+//  matching port (if TCP) or path (if IPC) exists, and if it does it will be
+//  set via the ZMQ_USE_FD socket option so that the library will use it
+//  instead of creating a new socket.
+
+
+void
+zsys_set_auto_use_fd (int auto_use_fd)
+{
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+    s_auto_use_fd = auto_use_fd;
+    ZMUTEX_UNLOCK (s_mutex);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Return use of automatic pre-allocated FDs for zsock instances.
+
+int
+zsys_auto_use_fd (void)
+{
+    return s_auto_use_fd;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Set log identity, which is a string that prefixes all log messages sent
 //  by this process. The log identity defaults to the environment variable
@@ -1396,7 +1585,7 @@ zsys_set_logsender (const char *endpoint)
             assert (s_logsender);
         }
         //  Bind/connect to specified endpoint(s) using zsock_attach() syntax
-	int rc = zsock_attach (s_logsender, endpoint, true);
+        int rc = zsock_attach (s_logsender, endpoint, true);
         assert (rc == 0);
     }
     else
@@ -1444,7 +1633,7 @@ s_log (char loglevel, char *string)
     else
     if (loglevel == 'D')
         priority = ANDROID_LOG_DEBUG;
-    
+
     __android_log_print(priority, "zsys", "%s", string);
 #   else
     if (s_logsystem) {
@@ -1469,10 +1658,6 @@ s_log (char loglevel, char *string)
     else
 #   endif
 #endif
-    //  Set s_logstream to stdout by default, unless we're using s_logsystem
-    if (!s_logstream)
-        s_logstream = stdout;
-
     if (s_logstream || s_logsender) {
         time_t curtime = time (NULL);
         struct tm *loctime = localtime (&curtime);
@@ -1505,7 +1690,7 @@ zsys_error (const char *format, ...)
     char *string = zsys_vprintf (format, argptr);
     va_end (argptr);
     s_log ('E', string);
-    free (string);
+    zstr_free (&string);
 }
 
 
@@ -1520,7 +1705,7 @@ zsys_warning (const char *format, ...)
     char *string = zsys_vprintf (format, argptr);
     va_end (argptr);
     s_log ('W', string);
-    free (string);
+    zstr_free (&string);
 }
 
 
@@ -1535,7 +1720,7 @@ zsys_notice (const char *format, ...)
     char *string = zsys_vprintf (format, argptr);
     va_end (argptr);
     s_log ('N', string);
-    free (string);
+    zstr_free (&string);
 }
 
 
@@ -1550,7 +1735,7 @@ zsys_info (const char *format, ...)
     char *string = zsys_vprintf (format, argptr);
     va_end (argptr);
     s_log ('I', string);
-    free (string);
+    zstr_free (&string);
 }
 
 
@@ -1565,7 +1750,7 @@ zsys_debug (const char *format, ...)
     char *string = zsys_vprintf (format, argptr);
     va_end (argptr);
     s_log ('D', string);
-    free (string);
+    zstr_free (&string);
 }
 
 
@@ -1591,8 +1776,6 @@ zsys_test (bool verbose)
         free (hostname);
         zsys_info ("system limit is %zu ZeroMQ sockets", zsys_socket_limit ());
     }
-    zsys_set_io_threads (1);
-    zsys_set_max_sockets (0);
     zsys_set_linger (0);
     zsys_set_sndhwm (1000);
     zsys_set_rcvhwm (1000);
@@ -1685,5 +1868,14 @@ zsys_test (bool verbose)
     zsys_close (logger, NULL, 0);
     //  @end
 
+    zsys_set_auto_use_fd (1);
+    assert (zsys_auto_use_fd () == 1);
+
+    assert (zsys_max_msgsz () == INT_MAX);
+    zsys_set_max_msgsz (2000);
+    assert (zsys_max_msgsz () == 2000);
+    zsys_set_max_msgsz (-1);
+    assert (zsys_max_msgsz () == 2000);
+
     printf ("OK\n");
 }
